@@ -239,8 +239,6 @@ struct svc_xprt {
 		svc_req_fun_t process_cb;
 		svc_xprt_fun_t rendezvous_cb;
 	}  xp_dispatch;
-	/* Handle resumed requests */
-	svc_req_fun_t xp_resume_cb;
 	SVCXPRT *xp_parent;
 
 	char *xp_tp;		/* transport provider device name */
@@ -273,6 +271,13 @@ struct svc_xprt {
 
 	int32_t xp_refcnt;	/* handle reference count */
 	uint16_t xp_flags;	/* flags */
+
+	union {
+		struct in_pktinfo in;
+#ifdef INET6
+		struct in6_pktinfo in6;
+#endif
+	} xp_pktinfo;
 };
 
 /* Service record used by exported search routines */
@@ -317,6 +322,10 @@ struct svc_req {
 	void *rq_ap1;		/* auth private */
 	void *rq_ap2;		/* auth private */
 
+	/* Handle resumed requests */
+	svc_req_fun_t rq_resume_cb;
+	struct work_pool_entry rq_wpe;
+
 	/* avoid separate alloc/free */
 	struct rpc_msg rq_msg;
 
