@@ -717,7 +717,7 @@ svc_rqst_rearm_events_locked(SVCXPRT *xprt, uint16_t ev_flags)
 		}
 
 		if (ev_flags & SVC_XPRT_FLAG_ADDED_SEND) {
-			ev = &rec->ev_u.epoll.event_recv;
+			ev = &rec->ev_u.epoll.event_send;
 
 			/* set up epoll user data */
 			ev->data.ptr = rec;
@@ -978,7 +978,7 @@ svc_rqst_evchan_write(SVCXPRT *xprt, struct xdr_ioq *xioq, bool has_blocked)
 				__warnx(TIRPC_DEBUG_FLAG_ERROR,
 					"%s: failed duplicating fd (%d)",
 					__func__, code);
-				goto out;
+				return (code);
 			}
 
 			__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
@@ -1001,25 +1001,15 @@ svc_rqst_evchan_write(SVCXPRT *xprt, struct xdr_ioq *xioq, bool has_blocked)
 					    SVC_XPRT_FLAG_ADDED_SEND);
 	}
 
-	if (code) {
+	if (unlikely(code)) {
 		__warnx(TIRPC_DEBUG_FLAG_ERROR,
-			"%s: failed hooking events (%d)",
+			"%s: failed evchan write control (%d)",
 			__func__, code);
-		goto out;
-	}
-
-	atomic_inc_int32_t(&sr_rec->ev_refcnt);
-	work_pool_submit(&svc_work_pool, &sr_rec->ev_wpe);
-
-	__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
-		"%s: create evchan write control fd pair (%d:%d)",
-		__func__,
-		sr_rec->sv[0], sr_rec->sv[1]);
-
-out:
-
-	if (code != 0) {
-		svc_rqst_release(sr_rec);
+	} else {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_RQST,
+			"%s: create evchan write control fd pair (%d:%d)",
+			__func__,
+			sr_rec->sv[0], sr_rec->sv[1]);
 	}
 
 	rpc_dplx_rui(rec);
@@ -1166,6 +1156,7 @@ svc_rqst_xprt_task_recv(struct work_pool_entry *wpe)
 	struct xdr_ioq *ioq =
 			opr_containerof(wpe, struct xdr_ioq, ioq_wpe);
 	struct rpc_dplx_rec *rec = ioq->rec;
+	enum xprt_stat stat = XPRT_IDLE;
 
 	atomic_clear_uint16_t_bits(&ioq->ioq_s.qflags, IOQ_FLAG_WORKING);
 
@@ -1183,11 +1174,13 @@ svc_rqst_xprt_task_recv(struct work_pool_entry *wpe)
 		 * xp_refcnt need more than 1 (this task).
 		 */
 		(void)clock_gettime(CLOCK_MONOTONIC_FAST, &rec->recv.ts);
-		(void)SVC_RECV(&rec->xprt);
+		stat = SVC_RECV(&rec->xprt);
 	}
 
-	/* If tests fail, log non-fatal "WARNING! already destroying!" */
-	SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
+	if (stat != XPRT_SUSPEND) {
+		/* If tests fail, log non-fatal "WARNING! already destroying!" */
+		SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
+	}
 }
 
 enum xprt_stat svc_request(SVCXPRT *xprt, XDR *xdrs)
@@ -1224,14 +1217,13 @@ enum xprt_stat svc_request(SVCXPRT *xprt, XDR *xdrs)
 
 static void svc_resume_task(struct work_pool_entry *wpe)
 {
-	struct rpc_dplx_rec *rec =
-			opr_containerof(wpe, struct rpc_dplx_rec, ioq.ioq_wpe);
-	struct svc_req *req = rec->svc_req;
-	SVCXPRT *xprt = &rec->xprt;
+	struct svc_req *req =
+			opr_containerof(wpe, struct svc_req, rq_wpe);
+	SVCXPRT *xprt = req->rq_xprt;
 	enum xprt_stat stat;
 
 	/* Resume the request. */
-	stat  = req->rq_xprt->xp_resume_cb(req);
+	stat  = req->rq_resume_cb(req);
 
 	if (stat == XPRT_SUSPEND) {
 		/* The rquest is suspended, don't touch the request in any way
@@ -1253,10 +1245,8 @@ static void svc_resume_task(struct work_pool_entry *wpe)
 
 void svc_resume(struct svc_req *req)
 {
-	struct rpc_dplx_rec *rpc_dplx_rec = REC_XPRT(req->rq_xprt);
-
-	rpc_dplx_rec->ioq.ioq_wpe.fun = svc_resume_task;
-	work_pool_submit(&svc_work_pool, &(rpc_dplx_rec->ioq.ioq_wpe));
+	req->rq_wpe.fun = svc_resume_task;
+	work_pool_submit(&svc_work_pool, &req->rq_wpe);
 }
 
 /*static*/ void
